# -*- coding: utf-8 -*-

import os
import json

from excel_reporting import report_gen

from kinder.core.audit_log.helpers import get_model

from concentrator.models import ChangeDeclaration


class ReportChanges(report_gen.BaseReport):
    """
    Генерирование заполненного заявления на изменение данных
    """

    def __init__(self, template, temp_filename, request, context):
        self.template_name = os.path.join(os.path.dirname(__file__), 'templates', template)
        self.result_name = temp_filename
        self.request = request
        self.context = context

    def collect(self, *args, **kwargs):
        if getattr(self.context, 'id', None):
            change = ChangeDeclaration.objects.get(
                id=self.context.id)

            def format_change(model, keys):
                def prepare(val):
                    if val == None:
                        return ' - '
                    else:
                        return unicode(val)

                return u"{0}: {1}".format(
                    model, u", ".join(map(lambda x: prepare(x), keys))
                )

            change_set = {
                'fields': [],
                'old_values': [],
                'new_values': []
            }

            for model, difference in json.loads(change.data).items():
                fields = []
                old_values = []
                new_values = []

                # пока игнорируем модели, потому что по ним
                # создается нестандартная диффка.
                if model in ['DeclarationPrivilege']:
                    continue

                model = get_model(model)._meta.verbose_name
                for field in difference:
                    [[field, new_value, old_value]] = field.values()
                    fields.append(field)
                    old_values.append(old_value)
                    new_values.append(new_value)

                change_set['fields'].append(
                    format_change(model, fields)
                )
                change_set['old_values'].append(
                    format_change(model, old_values)
                )
                change_set['new_values'].append(
                    format_change(model, new_values)
                )

            mo = change.declaration.mo
            children = change.declaration.children
            delegate = children.childrendelegate_set.all()[0].delegate

            row = {
                'director_fullname': mo.boss_fio,
                'delegate_fullname': delegate.fullname,
                'children_fullname': children.fullname,
                'birth_date': children.date_of_birth.strftime('%d.%m.%Y'),
                'fields': ', '.join(change_set['fields']),
                'old_values': ', '.join(change_set['old_values']),
                'new_values': ', '.join(change_set['new_values']),
                'date': change.create.strftime('%d.%m.%Y')
            }

            total = 1
        else:
            row = {}
            total = 0

        return {'rows': [row], 'total': total}
